Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var path = require('path');
var rollup = require('rollup');
var fs = require('fs');
var commonjs = require('@rollup/plugin-commonjs');
var shebang = require('rollup-plugin-preserve-shebang');
var json = require('@rollup/plugin-json');
var babel = require('@rollup/plugin-babel');
var nodeResolve = require('@rollup/plugin-node-resolve');
var rollupPluginTerser = require('rollup-plugin-terser');
require('arg');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
var shebang__default = /*#__PURE__*/_interopDefaultLegacy(shebang);
var json__default = /*#__PURE__*/_interopDefaultLegacy(json);
var babel__default = /*#__PURE__*/_interopDefaultLegacy(babel);
var nodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(nodeResolve);

var rootDir = process.cwd();
var config = {
    rootDir: rootDir,
};

function getPackageMeta() {
    var pkgFilePath = path__default["default"].resolve(config.rootDir, "package.json");
    var targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(fs__default["default"].readFileSync(pkgFilePath, { encoding: "utf-8" }));
    }
    catch (_) { }
    return targetPackageJson;
}
var logger = {
    log: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, args);
    },
    warn: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[33m"], args, false), ["\x1b[0m"], false));
    },
    error: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.error.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[31m"], args, false), ["\x1b[0m"], false));
    },
};

var Module = require("module").Module;
var hasLoggedTsWarning = false;
function resolveTypescript() {
    var ts;
    var m = new Module("", null);
    m.paths = Module._nodeModulePaths(config.rootDir);
    try {
        ts = m.require("typescript");
    }
    catch (_) {
        if (!hasLoggedTsWarning) {
            hasLoggedTsWarning = true;
            logger.warn("Could not load TypeScript compiler. Try `yarn add --dev typescript`");
        }
    }
    return ts;
}
function getDistPath(distPath) {
    return path.resolve(config.rootDir, distPath);
}
function createInputConfig(entry, pkg, options) {
    var _a;
    var externals = [pkg.peerDependencies, pkg.dependencies]
        .filter(function (n) { return Boolean(n); })
        .map(function (o) { return Object.keys(o); })
        .reduce(function (a, b) { return a.concat(b); }, [])
        .concat((_a = options.external) !== null && _a !== void 0 ? _a : []);
    var useTypescript = options.useTypescript, target = options.target, _b = options.minify, minify = _b === void 0 ? false : _b;
    var typings = pkg.types || pkg.typings;
    var cwd = config.rootDir;
    var plugins = [
        nodeResolve__default["default"]({
            preferBuiltins: target === "node",
            extensions: [".mjs", ".js", ".json", ".node", ".jsx"],
        }),
        commonjs__default["default"]({
            include: /node_modules\//,
        }),
        json__default["default"](),
        shebang__default["default"](),
        useTypescript && require("@rollup/plugin-typescript")(tslib.__assign({ tsconfig: (function () {
                var tsconfig = path.resolve(cwd, "tsconfig.json");
                return fs.existsSync(tsconfig) ? tsconfig : undefined;
            })(), typescript: resolveTypescript(), jsx: "react", module: "ES6", target: "ES5", noEmitOnError: !options.watch, sourceMap: options.sourcemap, declaration: !!typings }, (!!typings && {
            declarationDir: path.dirname(path.resolve(cwd, typings)),
        }))),
        !useTypescript && babel__default["default"]({
            babelHelpers: "bundled",
            babelrc: false,
            configFile: false,
            exclude: "node_modules/**",
            presets: [
                ["babel-preset-o", { targets: {} }]
            ],
        }),
        minify && rollupPluginTerser.terser({
            compress: {
                "keep_infinity": true,
            },
            format: {
                "comments": /^\s*([@#]__[A-Z]__\s*$|@[a-zA-Z]\s*$)/,
                "wrap_func_args": false,
                "preserve_annotations": true,
            }
        }),
    ].filter(function (n) { return Boolean(n); });
    return {
        input: entry,
        external: function (id) {
            return externals.some(function (name) { return id === name || id.startsWith(name + "/"); });
        },
        plugins: plugins,
        treeshake: {
            propertyReadSideEffects: false,
        },
        onwarn: function (warning, warn) {
            if (warning.code &&
                ['MIXED_EXPORTS', 'PREFER_NAMED_EXPORTS'].includes(warning.code))
                return;
            warn(warning);
        },
    };
}
function createOutputOptions(options, pkg) {
    var format = options.format, useTypescript = options.useTypescript;
    var tsCompilerOptions = {};
    if (useTypescript) {
        var ts = resolveTypescript();
        var tsconfigPath = path.resolve(config.rootDir, "tsconfig.json");
        if (fs__default["default"].existsSync(tsconfigPath)) {
            var tsconfigJSON = ts.readConfigFile(tsconfigPath, ts.sys.readFile).config;
            tsCompilerOptions = ts.parseJsonConfigFileContent(tsconfigJSON, ts.sys, "./").options;
        }
    }
    var exportPaths = getExportPaths(pkg);
    var useEsModuleMark = Boolean(tsCompilerOptions.esModuleInterop ||
        (exportPaths.main &&
            exportPaths.module));
    var file = path.resolve(options.file);
    return {
        name: pkg.name,
        dir: path.dirname(file),
        entryFileNames: path.basename(file),
        format: format,
        exports: "named",
        esModule: useEsModuleMark && format !== "umd",
        freeze: false,
        strict: false,
        sourcemap: options.sourcemap,
    };
}
function findExport(field) {
    if (!field)
        return null;
    if (typeof field === "string")
        return field;
    var value = field["."] || field["import"] || field["module"] || field["default"];
    return findExport(value);
}
function getExportPaths(pkg) {
    var paths = {
        main: null,
        module: null,
        export: null,
    };
    if (pkg.main) {
        paths.main = pkg.main;
    }
    if (pkg.module) {
        paths.module = pkg.module;
    }
    if (pkg.exports) {
        if (typeof pkg.exports === "string") {
            paths.export = pkg.exports;
        }
        else {
            paths.main = paths.main || pkg.exports["require"] || pkg.exports["node"] || pkg.exports["default"];
            paths.module = paths.module || pkg.exports["module"];
            paths.export = findExport(pkg.exports);
        }
    }
    return paths;
}
function getExportDist(pkg) {
    var paths = getExportPaths(pkg);
    var dist = [];
    if (paths.main) {
        dist.push({ format: "cjs", file: getDistPath(paths.main) });
    }
    if (paths.module) {
        dist.push({ format: "esm", file: getDistPath(paths.module) });
    }
    if (paths.export) {
        dist.push({ format: "esm", file: getDistPath(paths.export) });
    }
    if (dist.length === 0) {
        dist.push({ format: "cjs", file: getDistPath("dist/index.js") });
    }
    return dist;
}
function createRollupConfig(entry, pkg, cliArgs) {
    var file = cliArgs.file, format = cliArgs.format;
    var ext = path.extname(entry);
    var useTypescript = ext === ".ts" || ext === ".tsx";
    var options = tslib.__assign(tslib.__assign({}, cliArgs), { useTypescript: useTypescript });
    var inputOptions = createInputConfig(entry, pkg, options);
    var outputConfigs = getExportDist(pkg)
        .map(function (exportDist) {
        return createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), { file: exportDist.file, format: exportDist.format, useTypescript: useTypescript }), pkg);
    });
    if (file) {
        outputConfigs = [
            createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), { file: file, format: format, useTypescript: useTypescript }), pkg)
        ];
    }
    return {
        input: inputOptions,
        output: outputConfigs,
    };
}

function assignDefault(options, name, defaultValue) {
    if (!(name in options) || options[name] == null) {
        options[name] = defaultValue;
    }
}
function bundle(entry, _a) {
    if (_a === void 0) { _a = {}; }
    var cwd = _a.cwd, options = tslib.__rest(_a, ["cwd"]);
    config.rootDir = path.resolve(process.cwd(), cwd || "");
    assignDefault(options, "format", "es");
    assignDefault(options, "sourcemap", true);
    if (options.format === "esm") {
        options.format = "es";
    }
    var npmPackage = getPackageMeta();
    var rollupConfig = createRollupConfig(entry, npmPackage, options);
    if (options.watch) {
        return Promise.resolve(runWatch(rollupConfig));
    }
    return runBundle(rollupConfig);
}
function runWatch(_a) {
    var input = _a.input, output = _a.output;
    var watchOptions = [tslib.__assign(tslib.__assign({}, input), { output: output, watch: {
                exclude: ["node_modules/**"],
            } })];
    var watcher = rollup.watch(watchOptions);
    watcher.on('event', function (event) {
        if (event.code === 'ERROR') {
            onError(event.error);
        }
    });
    return watcher;
}
function runBundle(_a) {
    var input = _a.input, output = _a.output;
    return rollup.rollup(input).then(function (bundle) {
        var writeJobs = output.map(function (options) {
            return bundle.write(options);
        });
        return Promise.all(writeJobs);
    }, onError);
}
function onError(error) {
    if (!error)
        return;
    if (error.frame) {
        process.stdout.write(error.frame + "\n");
    }
    if (error.stack) {
        process.stdout.write(error.stack + "\n");
    }
    throw error;
}

exports.bundle = bundle;
//# sourceMappingURL=index.js.map
